/////////////////////////////////////////////////////////////////////////////////////////////
//
//	Callback Definitions
//
//	These individual callbacks may be used by Visual Basic Applications
//  to allow the WtHDAsvr.DLL to pass control whenever a client requests 
//  historical data.
//
//  C++ Applications will find it easier to override the CWtHDACallback()
//  object, rather than enable the following individual callbacks;
//
//  Used by the DLL to pass control back to the Server Application
//	Each callback must be explicitly enabled by calling one of the following
//	exported API functions:
//					EnableGetTimeStampCallback (TIMESTAMPPROC lpCallback);
//					EnableGetPreviousTimeStampCallback (PREVTIMESTAMPPROC lpCallback);
//					EnableGetNextTimeStampCallback (NEXTTIMESTAMPPROC lpCallback); 
//					EnableReadAtCallback (READATPROC lpCallback);
//					EnableReadCurrentAttributeCallback (CURRENTATTRIBUTEPROC lpCallback);
//					EnableReadAttributeAtCallback (READATTRIBUTEPROC lpCallback); 
//					EnablePrepareAggregatesCallback (PREPAREAGGREGATESPROC lpCallback);
//					EnableReadProcessedCallback (READPROCESSEDPROC lpCallback);
//		
//
//	TIMESTAMPPROC (HANDLE hItem, FILETIME *pStart, FILETIME *pEnd, int NumValues, FILETIME *pTimes, DWORD *pNumberOfValues)
//		Allows the DLL to retrieve a list of all the TimeStamps available
//		in the application database between the specified Start & End times.  As per the HDA
//		spec, the Start time may be greater than the End time, in which case TimeStamps must
//		be returned in reverse order.  If more data items exist in the specified time range
//		than can be filled in the allocated array, (limited by NumValues), the application
//		should return OPC_S_MOREDATA.  If the End time is not specified, (pEnd == NULL),
//		the application must use the Start time and NumValues to fill the TimeStamp array.  
//	PREVTIMESTAMPPROC (HANDLE hItem, FILETIME Time, FILETIME *pTimeStamp, DWORD *phResult)
//	NEXTTIMESTAMPPROC (HANDLE hItem, FILETIME Time, FILETIME *pTimeStamp, DWORD	*phResult)
//		PREVTIMESTAMPPROC and NEXTTIMESTAMPPROC allows the DLL to retrieve the HDA Bounding
//		value for a time period as requested by a client.  The application should return the
//		Timestamp of the historized item immediately prior to or immediately following the
//		specified FILETIME.
//	READATPROC  (HANDLE hItem, FILETIME TimeStamp, VARIANT *pValue, DWORD *pQuality, DWORD	*phResult)
//		READATPROC is used by the DLL to obtain the value and quality of an item at a specified time.
//		Generally, the requested timestamp will have been obtained by a previous call to
//		GetTimeStamps, however, the HDA spec allows a client to request the value of an item at
//		any given time.  In this case, the application should extrapolate the value for the 
//		item based on the surrounding data points and return OPCHDA_INTERPOLATED.  If the requested
//		timestamp is out of bounds of the database, the application should return OPC_S_NODATA.
//	CURRENTATTRIBUTEPROC (HANDLE hItem, DWORD AttributeID, FILETIME *pTime, VARIANT *pValue, DWORD *phResult)
//	READATTRIBUTEPROC (HANDLE hItem, DWORD AttributeID, FILETIME TimeStamp, VARIANT *pValue, DWORD *phResult)
//		When the application starts up, it specifies the attributes supported for 
//		the historized data items.  It also specifies, for each attribute, whether or
//		not the attribute is histroized along with the item.  These two callback functions
//		allow the dll to read the current value of an attribute for a given item and, if
//		supported, the value for an attribute at a specifiied point in time.
//	PREPAREAGGREGATESPROC (HANDLE hItem, DWORD AggregateID, FILETIME StartTime, FILETIME EndTime, FILETIME ReSampleInterval, DWORD *pNumberOfValues)
//		The PREPAREAGGREGATESPROC function allows the application to access the data and prepare
//		a list of calculated values as requested by an hda client.  The return value indicates
//		the number of calculated values, (Intervals), contained within the specified time range
//	READPROCESSEDPROC (HANDLE hItem, DWORD AggregateID, FILETIME *pStart, FILETIME *pEnd, VARIANT *pValue, DWORD *pQuality, DWORD *phResult)
//		The READPROCESSEDPROC allows the DLL to read a value from the application database
//		that is calculated over a specified time range.  The calculation to be performed on the
//		data set is defined by AggregateID.  The server application defines the list of
//		supported Aggregates during start-up.
/////////////////////////////////////////////////////////////////////////////////////////////
#ifdef STRICT
typedef VOID (CALLBACK* TIMESTAMPPROC)(HANDLE, FILETIME*, FILETIME*, DWORD, FILETIME*, DWORD*);
typedef VOID (CALLBACK* PREVTIMESTAMPPROC)(HANDLE, FILETIME, FILETIME*, HRESULT*);
typedef VOID (CALLBACK* NEXTTIMESTAMPPROC)(HANDLE, FILETIME, FILETIME*, HRESULT*);
typedef VOID (CALLBACK* READATPROC)(HANDLE, FILETIME, VARIANT*, DWORD*, HRESULT*);
typedef VOID (CALLBACK* CURRENTATTRIBUTEPROC)(HANDLE, DWORD, FILETIME*, VARIANT*, HRESULT*);
typedef VOID (CALLBACK* READATTRIBUTEPROC)(HANDLE, DWORD, FILETIME, VARIANT*, HRESULT*);
typedef VOID (CALLBACK* PREPAREAGGREGATESPROC)(HANDLE, DWORD, FILETIME, FILETIME, FILETIME, DWORD*);
typedef VOID (CALLBACK* READPROCESSEDPROC)(HANDLE, DWORD, FILETIME*, FILETIME*, VARIANT*, DWORD*, HRESULT*);
#else /* !STRICT */
typedef FARPROC TIMESTAMPPROC;
typedef FARPROC PREVTIMESTAMPPROC;
typedef FARPROC NEXTTIMESTAMPPROC;
typedef FARPROC READATPROC;
typedef FARPROC CURRENTATTRIBUTEPROC;
typedef FARPROC READATTRIBUTEPROC;
typedef FARPROC PREPAREAGGREGATESPROC;
typedef FARPROC READPROCESSEDPROC;
#endif



#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */


/////////////////////////////////////////////////////////////////////////////////////////////
//
//	Callback definition functions
//
/////////////////////////////////////////////////////////////////////////////////////////////


_declspec(dllexport) BOOL WINAPI EnableGetTimeStampCallback (TIMESTAMPPROC lpCallback);

_declspec(dllexport) BOOL WINAPI EnableGetPreviousTimeStampCallback (PREVTIMESTAMPPROC lpCallback);

_declspec(dllexport) BOOL WINAPI EnableGetNextTimeStampCallback (NEXTTIMESTAMPPROC lpCallback);

_declspec(dllexport) BOOL WINAPI EnableReadAtCallback (READATPROC lpCallback);

_declspec(dllexport) BOOL WINAPI EnableReadCurrentAttributeCallback (CURRENTATTRIBUTEPROC lpCallback);

_declspec(dllexport) BOOL WINAPI EnableReadAttributeAtCallback (READATTRIBUTEPROC lpCallback); 

_declspec(dllexport) BOOL WINAPI EnablePrepareAggregatesCallback (PREPAREAGGREGATESPROC lpCallback);

_declspec(dllexport) BOOL WINAPI EnableReadProcessedCallback (READPROCESSEDPROC lpCallback);


/////////////////////////////////////////////////////////////////////////////////////////////
//
//	Initialization and Registration functions
//
/////////////////////////////////////////////////////////////////////////////////////////////
//
//	InitWtHDAsvr1()
//		Initializes DCOM, Security, and creates the Server ClassFactory
//
//	HDAUpdateRegistry1()
//		Used by the application to install itself in the Windows Registry so
//		HDA Clients can find it.
//
//	UnregisterHDAServer1()
//		Un-Registers the server. Removes it from the Registry.
//
/////////////////////////////////////////////////////////////////////////////////////////////

_declspec(dllexport) BOOL WINAPI InitWtHDAsvr1 (BYTE *pCLSID_Svr, UINT ServerRate);

_declspec(dllexport) BOOL WINAPI HDAUpdateRegistry1 (BYTE *pCLSID_Svr, LPCSTR Name, LPCSTR Descr, LPCSTR ExePath);

_declspec(dllexport) BOOL WINAPI UnregisterHDAServer1 (BYTE *pCLSID_Svr, LPCSTR Name); 

/////////////////////////////////////////////////////////////////////////////////////////////
//
// HDAConvertVBDateToFileTime()
// HDAConvertFileTimeToVBDate()
//		To be used with Visual Basic to convert between the OPC timestamp
//		and a Date variable
//
////////////////////////////////////////////////////////////////////////////////////////////


_declspec(dllexport) BOOL WINAPI HDAConvertVBDateToFileTime (double *pVBDate, FILETIME *pFileTime);

_declspec(dllexport) BOOL WINAPI HDAConvertFileTimeToVBDate (FILETIME *pFileTime, double *pVBDate);




#ifdef __cplusplus
}
#endif
